/***************************************************************************
 *   Copyright (C) 2004 by David Banz                                      *
 *   neko@netcologne.de                                                    *
 *   GPL'ed                                                                *
 ***************************************************************************/

#include "KTfxWidget.h"
#include "ktfxsplash.h"

#include <qstrlist.h>
#include <qfileinfo.h>
#include <qtextstream.h>
#include <qtooltip.h>
#include <qtimer.h>
#include <qmessagebox.h>
#include <kmessagebox.h>

#include <kuniqueapp.h>
#include <kaboutapplication.h>
#include <kfiledialog.h>
#include <kurl.h>
#include <kconfig.h>
#include <klocale.h>

#include "sakuraplayer.xpm"

extern KUniqueApplication *g_pApp;
extern void sigHandler(int signal);

CFullGui::CFullGui(QWidget* parent, const char* name):CRawGui(parent, name)
{
	setIcon(sakuraplayer_xpm);
	setCaption("Sakuraplayer");
	
	m_pProc=0;		// no player-process object, yet
	m_pSplashScreen=0;	// no splash-screen either
	
	// check for tfmx-play first
	checkTfx();
	// do major part of i18n initialization
	i18nInitBulk();
	// set all intial defaults
	initParanoia();
	// read config file
	readConfig();
	// connect buttons
	buttonInit();
	// add m_pAccelerator for keyboard events and connect keys
	accInit();
	// update display with preloaded song et al., start player
	initPreload();
	
	// install signal handler
	signal (SIGINT, &sigHandler);
	signal (SIGTERM, &sigHandler);
	signal (SIGHUP, &sigHandler);
	signal (SIGQUIT, &sigHandler);
	signal (SIGABRT, &sigHandler);
	signal (SIGBUS, &sigHandler);
	signal (SIGSEGV, &sigHandler);

	// the following must be done AFTER readConfig() !
	if (m_bShowSplash)
	{
		// create the nice splash screen...
		CSplash* pSplashScreen=new CSplash(0, "Sakura Audio presents...");
		// ...let's show it...
		pSplashScreen->show();
		// ...and hide it after ten seconds
		QTimer::singleShot(10*1000, m_pSplashScreen, SLOT(hide()));
	}

	// activate DnD
	setAcceptDrops(true);
	
	// Greetings!
	std::cout << "Mina-san, konnichi wa! ^_^" << std::endl;
}

CFullGui::~CFullGui()
{
	std::cout << "Mata ne..." << std::endl;
}

void CFullGui::closeEvent(QCloseEvent*)
{
	quitTfx();	// close application and player when widget is closed
}

// ################ helper method for the signal handler to avoid segfaults ######
void CFullGui::signalHelper()
{
	quitTfx();	// close application and player when signal is caught
}

// ########## button slots #########

void CFullGui::stopTfx()
{
	// stop tfmx player if playing
	stopCore();
	saveConfig();
}

void CFullGui::quitTfx()
{
	// stop tfmx player
	stopTfx();
	// quit application
	g_pApp->quit();
}

void CFullGui::openTfx()
{
	// overloaded, call openTfx "without" a filename
	openTfx("");
}

void CFullGui::setFile(const QString& oFileName)
{
	QString oNewFile;
	// check for parameter
	if (m_nPlaying)
	{
		return;
	}
	
	if (oFileName.isEmpty())
	{
		return;
	}
			
	oNewFile = oFileName;
	QFileInfo oFi(oNewFile);	// to check for mdat file
	
	if (
		(!oNewFile.isEmpty())
		&&(oFi.isFile())&&(oFi.isReadable())
		&&(
		
		
		(
		((!(samplePath(oNewFile).isEmpty()))&&(oFi.fileName().left(5)=="mdat."))
		||
		((!(dosSamplePath(oNewFile).isEmpty()))&&(oFi.fileName().right(4)==".tfx"))
		)
		
		||(oFi.fileName().left(5)=="tfmx.")
		
		)
		)
	{
		stopTfx();
		// get new browse dir from dir path of selected file
		m_oBrowseDir = oFi.dirPath(true);
		m_oTfmxFile=oNewFile;
		m_nOpen=1;
		m_nSong=0;
		updateLcd();
	}
	saveConfig();
}

void CFullGui::openTfx(const QString& oFileName)
{
	QString oNewFile;
	// check for parameter
	if (!(oFileName.isEmpty()))
	{
		// if a filename is given, use it instead of opening a file browser
		oNewFile = oFileName;
	}
	else
	{
		// else open dialog to get tfmxFile
		oNewFile = KFileDialog::getOpenFileName(m_oBrowseDir, "mdat.* tfmx.* *.tfx", 0,
			"open tfmx file");
	}
	QFileInfo oFi(oNewFile);	// to check for mdat file
	
	if (
		(!oNewFile.isEmpty())
		&&(oFi.isFile())&&(oFi.isReadable())
		&&(
		
		
		(
		((!(samplePath(oNewFile).isEmpty()))&&(oFi.fileName().left(5)=="mdat."))
		||
		((!(dosSamplePath(oNewFile).isEmpty()))&&(oFi.fileName().right(4)==".tfx"))
		)
		
		||(oFi.fileName().left(5)=="tfmx.")
		
		)
		
		)
	{
		// stop player
		stopTfx();
		// get new browse dir from dir path of selected file
		m_oBrowseDir = oFi.dirPath(true);
		// start player
		m_oTfmxFile=oNewFile;
		m_nOpen=1;
		m_nSong=0;
		std::cout << "playing" << std::endl;
		playCore();
	}
	saveConfig();
}

void CFullGui::playStart()
{	
	if (!m_oTfmxFile.isEmpty()) // if a file has been loaded, play it
	{
		playTfx();
	}
}	

void CFullGui::setSong(int nNum)
{
	// if song is loaded but not playing, set the subsong
	if ((!m_nPlaying)&&(m_nOpen>0))
	{
		if ((nNum>=0)&&(nNum<=32))
		{
			m_nSong=nNum;
			updateLcd();
		}
	}
}

void CFullGui::playTfx()
{
	if (m_nPlaying==0)
	{
		if (m_nOpen==0)
		{
			openTfx();
		}
		else
		{
			playCore();
		}
	}
	saveConfig();
}

void CFullGui::songUp()
{
	if (m_nSong<31)
	{
		m_nSong++;
		if (m_nPlaying==1)
		{
			stopTfx();
			playTfx();
		}
		else updateLcd();
	}
	saveConfig();
}

void CFullGui::songDown()
{
	if (m_nSong>0)
	{
		m_nSong--;
		if (m_nPlaying==1)
		{
			stopTfx();
			playTfx();
		}
		else updateLcd();
	}
	saveConfig();
}
	
void CFullGui::bitsHi()
{
	if (m_nBits==8)
	{
		m_nBits=16;
		if (m_nPlaying==1)
		{
			stopTfx();
			playTfx();
		}
		else updateLcd();
	}	
	saveConfig();
}

void CFullGui::bitsLo()
{
	if (m_nBits==16)
	{
		m_nBits=8;
		if (m_nPlaying==1)
		{
			stopTfx();
			playTfx();
		}
		else updateLcd();
	}	
	saveConfig();
}

void CFullGui::freqUp()
{
	if (m_nFreq<44)
	{
		switch(m_nFreq)
		{
			case 32: m_nFreq=44;break;
			case 22: m_nFreq=32;break;
			case 16: m_nFreq=22;break;
			case 11: m_nFreq=16;break;
		}
		if (m_nPlaying==1)
		{
			stopTfx();
			playTfx();
		}
		else updateLcd();
	}	
	saveConfig();
}

void CFullGui::freqDown()
{
	if (m_nFreq>11)
	{
		switch(m_nFreq)
		{
			case 44: m_nFreq=32;break;
			case 32: m_nFreq=22;break;
			case 22: m_nFreq=16;break;
			case 16: m_nFreq=11;break;
		}
		if (m_nPlaying==1)
		{
			stopTfx();
			playTfx();
		}
		else updateLcd();
	}
	saveConfig();
}

// toggle oversampling + button
void CFullGui::toggleOsampButton()
{
	if (m_pOverSample->isChecked())
	{
		m_pOverSample->setChecked(false);
	}
	else
	{
		m_pOverSample->setChecked(true);
	}
	toggleOversampling();
}

// toggle gemx + button
void CFullGui::toggleGemXButton()
{
	if (m_pGemxButton->isChecked())
	{
		m_pGemxButton->setChecked(false);
	}
	else
	{
		m_pGemxButton->setChecked(true);
	}
	toggleGemX();
}

// toggle danger freak + button
void CFullGui::toggleDangerFreakButton()
{
	if (m_pDangerFreakButton->isChecked())
	{
		m_pDangerFreakButton->setChecked(false);
	}
	else
	{
		m_pDangerFreakButton->setChecked(true);
	}
	toggleDangerFreak();
}

void CFullGui::toggleGemX()
{
	if (m_nPlaying==1)
	{
		stopTfx();
		playTfx();
	}
}

void CFullGui::toggleDangerFreak()
{
	if (m_nPlaying==1)
	{
		stopTfx();
		playTfx();
	}
}

// toggle oversampling
void CFullGui::toggleOversampling()
{
	if (m_nPlaying==1)
	{
		stopTfx();
		playTfx();
	}
}

// modify channel separation...

void CFullGui::setMono()
{
	if (m_nSeparation!=0)
	{
		m_nSeparation=0;
		if (m_nPlaying==1)
		{
			stopTfx();
			playTfx();
		}
		else
		{
			updateLcd();
		}
		saveConfig();
	}
}

void CFullGui::setHeadphones()
{
	if (m_nSeparation!=1)
	{
		m_nSeparation=1;
		if (m_nPlaying==1)
		{
			stopTfx();
			playTfx();
		}
		else
		{
			updateLcd();
		}
		saveConfig();
	}
}

void CFullGui::setStereo()
{
	if (m_nSeparation!=2)
	{
		m_nSeparation=2;
		if (m_nPlaying==1)
		{
			stopTfx();
			playTfx();
		}
		else
		{
			updateLcd();
		}
		saveConfig();
	}
}

void CFullGui::addTipBulk()
{
	// the big block that adds all the tooltips
	QToolTip::add(m_pFileBar, m_oTipFileBar);
	QToolTip::add(m_pSongLcd, m_oTipSongLcd);
	QToolTip::add(m_pBitLcd, m_oTipBitLcd);
	QToolTip::add(m_pFreqLcd, m_oTipFreqLcd);
	QToolTip::add(m_pStatusBar, m_pTipStatusBar[m_nPlaying]);
	QToolTip::add(m_pPlayButton, m_oTipPlayButton);
	QToolTip::add(m_pOpenButton, m_oTipOpenButton);
	QToolTip::add(m_pStopButton, m_oTipStopButton);
	QToolTip::add(m_pQuitButton, m_oTipQuitButton);
	QToolTip::add(m_pDownButton, m_oTipDownButton);
	QToolTip::add(m_pUpButton, m_oTipUpButton);
	QToolTip::add(m_pEightButton, m_oTipEightButton);
	QToolTip::add(m_pSixteenButton, m_oTipSixteenButton);
	QToolTip::add(m_pHiButton, m_oTipHiButton);
	QToolTip::add(m_pLoButton, m_oTipLoButton);
	QToolTip::add(m_pToolTipCheck, m_oTipToolTipCheck);
	QToolTip::add(m_pHelpButton, m_oTipHelpButton);
	QToolTip::add(m_pAboutButton, m_oTipAboutButton);
	QToolTip::add(m_pMonoButton, m_oTipMonoButton);
	QToolTip::add(m_pHeadphonesButton, m_oTipHeadphonesButton);
	QToolTip::add(m_pStereoButton, m_oTipStereoButton);
	QToolTip::add(m_pSeparationBar, m_pTipSeparation[m_nSeparation]);	
	QToolTip::add(m_pOverSample, m_oTipOversampling);
	QToolTip::add(m_pGemxButton, m_oTipGemX);
	QToolTip::add(m_pDangerFreakButton, m_oTipDangerFreak);
}

void CFullGui::removeTipBulk()
{
	// the big block that removes all the tooltips
	QToolTip::remove(m_pFileBar);
	QToolTip::remove(m_pSongLcd);
	QToolTip::remove(m_pBitLcd);
	QToolTip::remove(m_pFreqLcd);
	QToolTip::remove(m_pStatusBar);
	QToolTip::remove(m_pPlayButton);
	QToolTip::remove(m_pOpenButton);
	QToolTip::remove(m_pStopButton);
	QToolTip::remove(m_pQuitButton);
	QToolTip::remove(m_pDownButton);
	QToolTip::remove(m_pUpButton);
	QToolTip::remove(m_pEightButton);
	QToolTip::remove(m_pSixteenButton);
	QToolTip::remove(m_pHiButton);
	QToolTip::remove(m_pLoButton);
	QToolTip::remove(m_pToolTipCheck);
	QToolTip::remove(m_pHelpButton);
	QToolTip::remove(m_pAboutButton);
	QToolTip::remove(m_pMonoButton);
	QToolTip::remove(m_pStereoButton);
	QToolTip::remove(m_pHeadphonesButton);
	QToolTip::remove(m_pSeparationBar);
	QToolTip::remove(m_pOverSample);
	QToolTip::remove(m_pGemxButton);
	QToolTip::remove(m_pDangerFreakButton);
}

void CFullGui::toggleTip()
{
	// turn tooltips off or on, depending on status of checkbox
	if (m_pToolTipCheck->isChecked())
	{
		addTipBulk();
	}
	else
	{
		removeTipBulk();
	}
	saveConfig();
}

void CFullGui::tipOff()
{
	// force tooltips off
	if (m_pToolTipCheck->isChecked())
	{
		removeTipBulk();
		m_pToolTipCheck->setChecked(false);
	}
	saveConfig();
}

void CFullGui::tipOn()
{
	// force tooltips on
	if (!m_pToolTipCheck->isChecked())
	{
		addTipBulk();
		m_pToolTipCheck->setChecked(true);
	}
	saveConfig();
}

void CFullGui::toggleCheckTip()
{
	// toggle checkbox and tooltips
	if (m_pToolTipCheck->isChecked())
	{
		tipOff();
	}
	else
	{
		tipOn();
	}
	saveConfig();
}

void CFullGui::help()
{
	// call help
	kapp->invokeHelp();	
}

void CFullGui::about()
{
	// create about box
	KAboutApplication* pAboutBox=new KAboutApplication(this);
	// construction done...let's show the about-box!
	pAboutBox->exec();
	// finally, remove the box (no point in setting it to 0)
	delete pAboutBox;
}
// ############### config file m_pAccess #######################

void CFullGui::readConfig()
{
	KConfig* pConfig = KGlobal::config();
	pConfig->setGroup(QString::fromLatin1("SakuraPlayer"));
	m_nBits=pConfig->readNumEntry(QString::fromLatin1("Bits"),16);
	m_nFreq=pConfig->readNumEntry(QString::fromLatin1("Freq"),44);
	m_oBrowseDir=pConfig->readEntry(QString::fromLatin1("BrowseDir"),0);
	m_oPreloadSong=pConfig->readEntry(QString::fromLatin1("RecentFile"),0);
	m_nSong=m_nPreloadSubsong=pConfig->readNumEntry(QString::fromLatin1("RecentSong"),0);
	int nBoolTip=pConfig->readNumEntry(QString::fromLatin1("Tooltips"),false);	
	m_nSeparation=pConfig->readNumEntry(QString::fromLatin1("Separation"),1);
	int nBoolSample=pConfig->readNumEntry(QString::fromLatin1("Oversampling"),false);
	int nMajor=pConfig->readNumEntry(QString::fromLatin1("Major"),-1);
	int nMinor=pConfig->readNumEntry(QString::fromLatin1("Minor"),-1);
	int nPatch=pConfig->readNumEntry(QString::fromLatin1("Patch"),-1);
	int nBoolGemX=pConfig->readNumEntry(QString::fromLatin1("GemX"),false);
	int nBoolDangerFreak=pConfig->readNumEntry(QString::fromLatin1("DangerFreak"),false);
	
	// determine whether to show the splash screen...
	m_bShowSplash=false;
	if ((nMajor*nMinor*nPatch) < 0)
	{
		m_bShowSplash=true;
	}
	else if (MAJOR>nMajor)
	{
		m_bShowSplash=true;
	}
	else if (MINOR>nMinor)
	{
		m_bShowSplash=true;
	}
	else if (PATCH>nPatch)
	{
		m_bShowSplash=true;
	}
	
	// set oversampling
	if (nBoolSample)
	{
		m_pOverSample->setChecked(true);
	}
	else
	{
		m_pOverSample->setChecked(false);
	}
		
	// set gemx
	if (nBoolGemX)
	{
		m_pGemxButton->setChecked(true);
	}
	else
	{
		m_pGemxButton->setChecked(false);
	}
	
	// set danger freak
	if (nBoolDangerFreak)
	{
		m_pDangerFreakButton->setChecked(true);
	}
	else
	{
		m_pDangerFreakButton->setChecked(false);
	}
	
	// here we set the tooltips m_pAccording to the config file
	if (nBoolTip)
	{
		tipOn();
	}
	else
	{
		tipOff();
	}
	
	// now we check whether the file from the config file really exists
	QFileInfo oFiPre(m_oPreloadSong);

	if (!((oFiPre.exists())&&(oFiPre.isFile())&&(oFiPre.isReadable())&&
		((oFiPre.fileName().left(5)=="mdat.")
		||(oFiPre.fileName().right(4)==".tfx")
		||(oFiPre.fileName().left(5)=="tfmx."))))
	{
		m_oPreloadSong="";	// reset preload song
		m_oBrowseDir="";	// reset browse dir
		m_nSong=0;		// reset subsong
	}
}

void CFullGui::saveConfig()
{
	// update to KDE2 standards

	// get new browse dir from dir. path of current songfile
	QFileInfo oFi(m_oTfmxFile);
	m_oBrowseDir = oFi.dirPath(true);
	
	KConfig* pConfig = KGlobal::config();
	pConfig->setGroup(QString::fromLatin1("SakuraPlayer"));
	pConfig->writeEntry(QString::fromLatin1("Bits"),this->m_nBits);
	pConfig->writeEntry(QString::fromLatin1("Freq"),this->m_nFreq);
	pConfig->writeEntry(QString::fromLatin1("BrowseDir"),this->m_oBrowseDir);
	pConfig->writeEntry(QString::fromLatin1("RecentFile"),this->m_oTfmxFile);
	pConfig->writeEntry(QString::fromLatin1("RecentSong"),this->m_nSong);
	pConfig->writeEntry(QString::fromLatin1("Tooltips"),
		(int)this->m_pToolTipCheck->isChecked());
	pConfig->writeEntry(QString::fromLatin1("Separation"),this->m_nSeparation);
	pConfig->writeEntry(QString::fromLatin1("Oversampling"),
		(int)this->m_pOverSample->isChecked());
	pConfig->writeEntry(QString::fromLatin1("GemX"),(int)this->m_pGemxButton->isChecked());
	pConfig->writeEntry(QString::fromLatin1("DangerFreak"),
		(int)this->m_pDangerFreakButton->isChecked());
	pConfig->writeEntry(QString::fromLatin1("Major"),MAJOR);
	pConfig->writeEntry(QString::fromLatin1("Minor"),MINOR);
	pConfig->writeEntry(QString::fromLatin1("Patch"),PATCH);
	pConfig->sync();
}

void CFullGui::checkTfx()
{
	// initilialize translation for error box:
	// we don't do this in i18nInitBulk(), because if an error occurs right here,
	// there is no need to go through the rest of the i18n code anyway
	m_pNoTfxStr[0]=i18n("Error!");
	m_pNoTfxStr[1]=i18n("The required binary tfmx-play cannot not");
	m_pNoTfxStr[2]=i18n("be executed. Check path/permissions.");
	m_pNoTfxStr[3]=i18n("Quit");	// this is the button text!
}

// ################ initially connect buttons ###################

void CFullGui::buttonInit()
{
	connect(m_pQuitButton, SIGNAL(clicked()), this, SLOT(quitTfx()));
	connect(m_pPlayButton, SIGNAL(clicked()), this, SLOT(playTfx()));
	connect(m_pStopButton, SIGNAL(clicked()), this, SLOT(stopTfx()));
	connect(m_pOpenButton, SIGNAL(clicked()), this, SLOT(openTfx()));
	
	connect(m_pUpButton, SIGNAL(clicked()), this, SLOT(songUp()));
	connect(m_pDownButton, SIGNAL(clicked()), this, SLOT(songDown()));
	
	connect(m_pEightButton, SIGNAL(clicked()), this, SLOT(bitsLo()));
	connect(m_pSixteenButton, SIGNAL(clicked()), this, SLOT(bitsHi()));
	
	connect(m_pHiButton, SIGNAL(clicked()), this, SLOT(freqUp()));
	connect(m_pLoButton, SIGNAL(clicked()), this, SLOT(freqDown()));

	connect(m_pToolTipCheck, SIGNAL(clicked()), this, SLOT(toggleTip()));

	connect(m_pHelpButton, SIGNAL(clicked()), this, SLOT(help()));
	connect(m_pAboutButton, SIGNAL(clicked()), this, SLOT(about()));

	connect(m_pStereoButton, SIGNAL(clicked()), this, SLOT(setStereo()));
	connect(m_pHeadphonesButton, SIGNAL(clicked()), this, SLOT(setHeadphones()));
	connect(m_pMonoButton, SIGNAL(clicked()), this, SLOT(setMono()));
	
	connect(m_pOverSample, SIGNAL(clicked()), this, SLOT(toggleOversampling()));
	
	connect(m_pGemxButton, SIGNAL(clicked()), this, SLOT(toggleGemX()));
	connect(m_pDangerFreakButton, SIGNAL(clicked()), this, SLOT(toggleDangerFreak()));
}


// ################# accelerator stuff ########################

void CFullGui::accInit()
{
	m_pAcc=new QAccel(this);
	
	m_pAcc->connectItem(m_pAcc->insertItem(Key_H), this, SLOT(help()));
 	m_pAcc->connectItem(m_pAcc->insertItem(Key_A), this, SLOT(about()));
	
	m_pAcc->connectItem(m_pAcc->insertItem(Key_P), this, SLOT(playTfx()));
	m_pAcc->connectItem(m_pAcc->insertItem(Key_S), this, SLOT(stopTfx()));
	m_pAcc->connectItem(m_pAcc->insertItem(Key_O), this, SLOT(openTfx()));
	m_pAcc->connectItem(m_pAcc->insertItem(Key_Q), this, SLOT(quitTfx()));

	m_pAcc->connectItem(m_pAcc->insertItem(Key_1), this, SLOT(songUp()));
	m_pAcc->connectItem(m_pAcc->insertItem(Key_2), this, SLOT(songDown()));
	m_pAcc->connectItem(m_pAcc->insertItem(Key_3), this, SLOT(bitsHi()));
	m_pAcc->connectItem(m_pAcc->insertItem(Key_4), this, SLOT(bitsLo()));
	m_pAcc->connectItem(m_pAcc->insertItem(Key_5), this, SLOT(freqUp()));
 	m_pAcc->connectItem(m_pAcc->insertItem(Key_6), this, SLOT(freqDown()));

 	m_pAcc->connectItem(m_pAcc->insertItem(Key_T), this, SLOT(toggleCheckTip()));

	m_pAcc->connectItem(m_pAcc->insertItem(Key_8), this, SLOT(setStereo()));
	m_pAcc->connectItem(m_pAcc->insertItem(Key_9), this, SLOT(setHeadphones()));
	m_pAcc->connectItem(m_pAcc->insertItem(Key_0), this, SLOT(setMono()));
 	
	m_pAcc->connectItem(m_pAcc->insertItem(Key_V), this, SLOT(toggleOsampButton()) );
	
	m_pAcc->connectItem(m_pAcc->insertItem(Key_G), this, SLOT(toggleGemXButton()) );
	m_pAcc->connectItem(m_pAcc->insertItem(Key_D), this, SLOT(toggleDangerFreakButton()) );
	
	// "scroll up/down" is here interpreted as "previous/next subsong"
	m_pAcc->connectItem(m_pAcc->insertItem(Key_F1), this, SLOT(help()));
	m_pAcc->connectItem(m_pAcc->insertItem(CTRL+Key_O), this, SLOT(openTfx()));
	m_pAcc->connectItem(m_pAcc->insertItem(CTRL+Key_Q), this, SLOT(quitTfx()));
	m_pAcc->connectItem(m_pAcc->insertItem(Key_Next), this, SLOT(songUp()));
	m_pAcc->connectItem(m_pAcc->insertItem(Key_Prior), this, SLOT(songDown()));
}

// ################# set all defaults ########################

void CFullGui::initParanoia()
{
	m_nBits=16;	// 8 or 16 bit
	m_nFreq=44;	// 44=44100 Hz, 32=32000 Hz,  22=22050 Hz, 16=16000 Hz, 11=10025 Hz
	m_nSong=0;	// subsong number
	m_nOpen=0;	// song open?
	m_nPlaying=0;	// song playing?
	m_oBrowseDir="";	// start browsing in current dir
	m_oPreloadSong="";	// song to preload
	m_nPreloadSubsong=0;	// subsong to preselect
	m_oTfmxFile="";		// mdat song filename
	m_pToolTipCheck->setChecked(false);	// no tooltips, so we uncheck this button
	m_pOverSample->setChecked(false);	// no oversampling
	m_pGemxButton->setChecked(false);		// no gemx hack
	m_pDangerFreakButton->setChecked(false);	// no danger freak hack
}

// #### return path of sample file for given mdat.-file, or empty string if it does not exist
QString CFullGui::samplePath(const QString& oMdatPath)
{
			QFileInfo oFiMd(oMdatPath);
			QString oSmplFile=oFiMd.fileName();
			// mdat.somefile
			QString oSmplPath=oMdatPath;
			// /where/ever/it/is/mdat.somefile
			oSmplPath=oSmplPath.left(oSmplPath.length()-oSmplFile.length());
			// /where/ever/it/is/
			oSmplFile=oSmplFile.remove(0,5);
			// somefile
			oSmplFile="smpl."+oSmplFile;
			// smpl.somefile
			oSmplPath=oSmplPath+oSmplFile;
			// /where/ever/it/is/smpl.somefile

			QFileInfo oFiSm(oSmplPath);
			if ((oFiSm.exists())&&(oFiSm.isFile())
				&&(oFiSm.isReadable())&&(oFiSm.fileName().left(5)=="smpl."))
			{
				return oSmplPath;
			}
			return "";
}

// #### return path of sample file for given .tfx-file, or empty string if it does not exist
QString CFullGui::dosSamplePath(const QString& oMdatPath)
{
			QFileInfo oFiMd(oMdatPath);
			QString oSmplFile=oFiMd.fileName();
			// somefile.tfx
			QString oSmplPath=oMdatPath;
			// /where/ever/it/is/somefile.tfx
			oSmplPath=oSmplPath.left(oSmplPath.length()-oSmplFile.length());
			// /where/ever/it/is/
			oSmplFile=oSmplFile.remove(oSmplFile.length()-3,3);
			// somefile
			oSmplFile=oSmplFile+"sam";
			// somefile.sam
			oSmplPath=oSmplPath+oSmplFile;
			// /where/ever/it/is/somefile.sam

			QFileInfo oFiSm(oSmplPath);
			if ((oFiSm.exists())&&(oFiSm.isFile())
				&&(oFiSm.isReadable())&&(oFiSm.fileName().right(4)==".sam"))
			{
				return oSmplPath;
			}
			return "";
}

// ######## update display with preloaded song et al., start player ###########
void CFullGui::initPreload()
{
	if (!(m_oPreloadSong.isEmpty()))	// show preloaded songfile
	{
		m_oTfmxFile=m_oPreloadSong;
		m_nOpen=1;
		m_pFileBar->storeText(m_oTfmxFile);
		m_pSongLcd->display(m_nSong);
	}
	m_pFreqLcd->display(m_nFreq);	// sound settings
	m_pBitLcd->display(m_nBits);
}

// ################## cleanup the player process ##############
void CFullGui::done(KProcess*)
{
	// inform the user that root may not run tfmx-play
	if (m_pProc->normalExit())
	{
		if (133 == m_pProc->exitStatus())
		{
			// root wanted to start TFMX-Play
			KMessageBox::sorry(this, i18n("The player may not be started by root."));
		}
	}

	// remove KProcess instance
	delete m_pProc;
	m_pProc=0;
	// make sure the frontend really knows that the player is gone
	// (this is needed when /dev/dsp is busy, for example, or when
	// the player is killed from outside)
	m_pStatusBar->setText(i18n("stopped"));
	m_nPlaying=0;
	if (m_pToolTipCheck->isChecked())
	{	// update tooltip, if active
		QToolTip::remove(m_pStatusBar);
		QToolTip::add(m_pStatusBar, m_pTipStatusBar[m_nPlaying]);
	}
}

// ############# core stuff to stop the player #################

void CFullGui::stopCore()
{
	if (m_nPlaying!=0)
	{
		// kill player process
		if (m_pProc)
		{
			if (m_pProc->isRunning())
			{
				m_pProc->kill();
			}
		}
		
		m_pStatusBar->setText(i18n("stopped"));
		m_nPlaying=0;
		if (m_pToolTipCheck->isChecked())
		{	// update tooltip, if active
			QToolTip::remove(m_pStatusBar);
			QToolTip::add(m_pStatusBar, m_pTipStatusBar[m_nPlaying]);
		}
	}
}

// ########### core stuff to start the player ############

void CFullGui::playCore()
{
	QString oSongStr;
	oSongStr.setNum(m_nSong);

	QString oFreqStr;
	switch(m_nFreq)
	{
		case 44:
			oFreqStr=" -f44100 ";break;
		case 32:
			oFreqStr=" -f32000 ";break;
		case 22:
			oFreqStr=" -f22050 ";break;
		case 16:
			oFreqStr=" -f16000 ";break;
		case 11:
			oFreqStr=" -f11050 ";break;
	}

	QString oBitStr;
	switch(m_nBits)
	{
		case 16:
			oBitStr="";break;
		case 8:
			oBitStr=" -8 ";break;
	}

	QString oSepStr;
	oSepStr.setNum(m_nSeparation);

	updateLcd();

	m_pFileBar->storeText(m_oTfmxFile);

	delete m_pProc;
	m_pProc=new KShellProcess("bash");
	
	if (m_pProc)
	{
		// clear all arguments
		m_pProc->clearArguments();

		*m_pProc << "tfmx-play";	// set executable name first
		*m_pProc << m_oTfmxFile;
		*m_pProc << "-S";
		*m_pProc << "-p" << oSongStr << oBitStr << oFreqStr;
		*m_pProc << "-b" << oSepStr;

		// turn oversampling off, if box is not checked
		if (!m_pOverSample->isChecked())
		{
			*m_pProc << "-v";
		}
		// turn gemx hack on, if box is checked
		if (m_pGemxButton->isChecked())
		{
			*m_pProc << "-G";
		}
		// turn danger freak hack on, if box is checked
		if (m_pDangerFreakButton->isChecked())
		{
			*m_pProc << "-D";
		}
		// player output MUST NOT be sent to /dev/null for KShellProcess to work
	}
	// connect "done"-slot to KProcess instance
	QApplication::connect(m_pProc, SIGNAL(processExited(KProcess *)),
		this, SLOT(done(KProcess*)));

	// try to start player
	if (m_pProc->start(KShellProcess::NotifyOnExit,KShellProcess::NoCommunication))
	{
		m_pStatusBar->setText(i18n("playing"));
		m_nPlaying=1;
	}
	else
	{
		m_pStatusBar->setText(i18n("stopped"));
		m_nPlaying=0;
	}
	
	if (m_pToolTipCheck->isChecked())
	{	// update tooltips, if active
		QToolTip::remove(m_pStatusBar);
		QToolTip::add(m_pStatusBar, m_pTipStatusBar[m_nPlaying]);
	}
}

void CFullGui::updateLcd()
{
	// show separation (no LCD, but who cares)
	m_pSeparationBar->setText(m_pShowSeparation[m_nSeparation]);
	if (m_pToolTipCheck->isChecked())
	{	// update tooltips, if active
		QToolTip::remove(m_pSeparationBar);
		QToolTip::add(m_pSeparationBar, m_pTipSeparation[m_nSeparation]);
	}
	// the real LCDs
	m_pFreqLcd->display(m_nFreq);
	m_pBitLcd->display(m_nBits);
	m_pSongLcd->display(m_nSong);
}

void CFullGui::i18nInitBulk()
{
	// initialize translated tooltips
	m_oTipFileBar=i18n("Loaded songfile");
	m_oTipSongLcd=i18n("Selected sub-song");
	m_oTipFreqLcd=i18n("Selected frequency");
	m_oTipBitLcd=i18n("Selected bits per sample");
	m_pTipStatusBar[0]=i18n("Status: stopped");
	m_pTipStatusBar[1]=i18n("Status: playing");
	m_oTipPlayButton=i18n("Play");
	m_oTipStopButton=i18n("Stop");
	m_oTipOpenButton=i18n("Open");
	m_oTipQuitButton=i18n("Quit");
	m_oTipUpButton=i18n("Next sub-song");
	m_oTipDownButton=i18n("Previous sub-song");
	m_oTipEightButton=i18n("8 bits per sample");
	m_oTipSixteenButton=i18n("16 bits sample");
	m_oTipHiButton=i18n("Raise frequency");
	m_oTipLoButton=i18n("Lower frequency");
	m_oTipHelpButton=i18n("Help");
	m_oTipAboutButton=i18n("About");
	m_oTipToolTipCheck=i18n("Tooltips");
	m_pTipSeparation[0]=i18n("Channel separation: Mono");
	m_pTipSeparation[1]=i18n("Channel separation: Stereo for headphones");
	m_pTipSeparation[2]=i18n("Channel separation: Default stereo");
	m_oTipMonoButton=i18n("Mono");
	m_oTipStereoButton=i18n("Default stereo (hard channel separation)");
	m_oTipHeadphonesButton=i18n("Stereo for headphones (overlapping channels)");
	m_oTipOversampling=i18n("Enhanced mixing quality");
	m_oTipGemX=i18n("Apply (incomplete) fix for the GemX title\nto all other songs as well");
	m_oTipDangerFreak=i18n(
		"Apply fix for the Danger Freak title\nto all other song as well");
	// initialize "about"-box translation
	m_pAboutStr[0]=i18n("Sakuraplayer Version ");
	m_pAboutStr[1]=i18n("KDE3-based frontend for TFMX-PLAY");
	m_pAboutStr[2]=i18n("Author: David Banz");
	m_pAboutStr[3]=i18n("neko@netcologne.de");
	// initialize translations for the buttons
	m_pEightButton->setText(i18n("8 bit"));
	m_pSixteenButton->setText(i18n("16 bit"));
	m_pHiButton->setText(i18n("Freq+"));
	m_pLoButton->setText(i18n("Freq-"));
	m_pToolTipCheck->setText(i18n("Tooltips"));
	m_pOverSample->setText(i18n("Oversampling"));
	// initialize translation for the channel separation display
	m_pShowSeparation[0]=i18n("Mono");
	m_pShowSeparation[1]=i18n("Headphones");
	m_pShowSeparation[2]=i18n("Stereo");
	m_pSeparationBar->setText(m_pShowSeparation[1]);

	// ######### that's it for most of the translation... ########
}

void CFullGui::dragEnterEvent(QDragEnterEvent *e)
{
	// check if it is a URI whe can use
	e->accept(QUriDrag::canDecode(e));
}

void CFullGui::dropEvent(QDropEvent *e)
{
	QStrList oFileList;
	// get list of dropped files
	if(QUriDrag::decode(e, oFileList))
	{
		// get first file in list
		KURL oUrl(oFileList.at(0));
		if(oUrl.isValid())
		{
			// try to open the dropped file
			openTfx(oUrl.path());
		}
	}
}

// ######################### OWARI / THE END ############################
