/***************************************************************************
 *   Copyright (C) 2004 by David Banz                                      *
 *   neko@netcologne.de                                                    *
 *   GPL'ed                                                                *
 ***************************************************************************/

#include <kcmdlineargs.h>
#include <kaboutdata.h>
#include <klocale.h>
#include <kuniqueapp.h>
#include <kaboutapplication.h>
#include "KTfxWidget.h"

KUniqueApplication *g_pApp;
CFullGui *g_pGui;

// ########## signal handler (makes sure the player is closed as well) ##########
void sigHandler(int n_signal)
{
	// something awful may have happened, tell the user on STDERR
	if(n_signal==SIGSEGV)
	{	
		std::cerr
			<< "Sakuraplayer caught SIGSEGV (segmentation fault) - exiting..."
			<< std::endl;
	}
	else if(n_signal==SIGBUS)
	{
		std::cerr
			<< "Sakuraplayer caught SIGSBUS (bus error) - exiting..."
			<< std::endl;
	}
	// this makes sure the player is stopped as well
	g_pGui->signalHelper();
}

static const char *description =
	I18N_NOOP("Player for TFMX music files.\nRelease: "INTERNAL_DESIGNATION);
	
static KCmdLineOptions options[] =
{
	{  "+[mdat-file]", I18N_NOOP("Specifies the TFMX .mdat file to open."), 0  },
	{  "+[sub-song]", I18N_NOOP("Specifies the sub-song to play."), 0  },
	{  0, 0, 0  }
};

int main(int argc, char *argv[])
{
	KAboutData aboutData( "sakuraplayer", I18N_NOOP("Sakuraplayer"),
		VERSION, description, KAboutData::License_GPL,
		"(c) 2004, David Banz",
		"visit http://exotica.fix.no for TFMX tunes",
		"http://darkstar.tabu.stw-bonn.de/~neo/audio.html",
		"neko@netcologne.de");
	aboutData.addAuthor("David Banz",
		"Developer and anime-fan ... ^_^",
		"neko@netcologne.de",
		"http://darkstar.tabu.stw-bonn.de/~neo/");
	KCmdLineArgs::init( argc, argv, &aboutData );
	KCmdLineArgs::addCmdLineOptions( options );
	KUniqueApplication::addCmdLineOptions();
	
	if (!KUniqueApplication::start())
	{
		std::cerr << "Another instance of Sakuraplayer is already running - exiting." << std::endl << std::endl;
		exit(0);
	}
	
	KUniqueApplication oApp;
	g_pApp=&oApp;
	CFullGui* sakuraplayer = new CFullGui(0,"Sakuraplayer");
	g_pGui=sakuraplayer;
	oApp.setMainWidget(sakuraplayer);
 	sakuraplayer->show();

	KCmdLineArgs *pArgs = KCmdLineArgs::parsedArgs();
	
	if (pArgs->count()>0)
	{
		sakuraplayer->setFile(pArgs->arg(0));	// set songfile
	}
	if (pArgs->count()>1)
	{
		QString	oNum=pArgs->arg(1);
		sakuraplayer->setSong(oNum.toInt());	// set subsong
	}	
	sakuraplayer->playStart();
	pArgs->clear();
	
	return oApp.exec();
}
